#!/usr/bin/env python 
# -*- coding: utf-8 -*-
# @Time    : 2019-12-10 15:33
# @Author  : Luke
# @File    : utils.py
# @Desc    :

import uuid
import re
from urllib.parse import unquote, parse_qsl
import collections.abc as collections_abc


class URL(object):
    """
    Represent the components of a URL used to connect to a database.
    This object is suitable to be passed directly to a
    :func:`~sqlalchemy.create_engine` call.  The fields of the URL are parsed
    from a string by the :func:`.make_url` function.  the string
    format of the URL is an RFC-1738-style string.
    All initialization parameters are available as public attributes.
    :param drivername: the name of the database backend.
      This name will correspond to a module in sqlalchemy/databases
      or a third party plug-in.
    :param username: The user name.
    :param password: database password.
    :param host: The name of the host.
    :param port: The port number.
    :param database: The database name.
    :param query: A dictionary of options to be passed to the
      dialect and/or the DBAPI upon connect.
    """

    def __init__(
            self,
            drivername,
            username=None,
            password=None,
            host=None,
            port=None,
            database=None,
            query=None,
    ):
        self.drivername = drivername
        self.username = username
        self.password_original = password
        self.host = host
        if port is not None:
            self.port = int(port)
        else:
            self.port = None
        self.database = database
        self.query = query or {}

    def __to_string__(self, hide_password=True):
        s = self.drivername + "://"
        if self.username is not None:
            s += _rfc_1738_quote(self.username)
            if self.password is not None:
                s += ":" + (
                    "***" if hide_password else _rfc_1738_quote(self.password)
                )
            s += "@"
        if self.host is not None:
            if ":" in self.host:
                s += "[%s]" % self.host
            else:
                s += self.host
        if self.port is not None:
            s += ":" + str(self.port)
        if self.database is not None:
            s += "/" + self.database
        if self.query:
            keys = list(self.query)
            keys.sort()
            s += "?" + "&".join(
                "%s=%s" % (k, element)
                for k in keys
                for element in to_list(self.query[k])
            )
        return s

    def __str__(self):
        return self.__to_string__(hide_password=False)

    def __repr__(self):
        return self.__to_string__()

    def __hash__(self):
        return hash(str(self))

    def __eq__(self, other):
        return (
                isinstance(other, URL)
                and self.drivername == other.drivername
                and self.username == other.username
                and self.password == other.password
                and self.host == other.host
                and self.database == other.database
                and self.query == other.query
                and self.port == other.port
        )

    def __ne__(self, other):
        return not self == other

    @property
    def password(self):
        if self.password_original is None:
            return None
        else:
            return str(self.password_original)

    @password.setter
    def password(self, password):
        self.password_original = password

    def get_backend_name(self):
        if "+" not in self.drivername:
            return self.drivername
        else:
            return self.drivername.split("+")[0]

    def translate_connect_args(self, names=[], **kw):
        r"""Translate url attributes into a dictionary of connection arguments.
        Returns attributes of this url (`host`, `database`, `username`,
        `password`, `port`) as a plain dictionary.  The attribute names are
        used as the keys by default.  Unset or false attributes are omitted
        from the final dictionary.
        :param \**kw: Optional, alternate key names for url attributes.
        :param names: Deprecated.  Same purpose as the keyword-based alternate
            names, but correlates the name to the original positionally.
        """

        translated = {}
        attribute_names = ["host", "database", "username", "password", "port"]
        for sname in attribute_names:
            if names:
                name = names.pop(0)
            elif sname in kw:
                name = kw[sname]
            else:
                name = sname
            if name is not None and getattr(self, sname, False):
                translated[name] = getattr(self, sname)
        return translated


def to_list(x, default=None):
    if x is None:
        return default
    if not isinstance(x, collections_abc.Iterable) or isinstance(
        x, (str, ) + (bytes,)
    ):
        return [x]
    elif isinstance(x, list):
        return x
    else:
        return list(x)


def make_url(name_or_url):
    """Given a string or unicode instance, produce a new URL instance.
    The given string is parsed according to the RFC 1738 spec.  If an
    existing URL object is passed, just returns the object.
    """
    return _parse_rfc1738_args(name_or_url)


def _parse_rfc1738_args(name):
    pattern = re.compile(
        r"""
            (?P<name>[\w\+]+)://
            (?:
                (?P<username>[^:/]*)
                (?::(?P<password>.*))?
            @)?
            (?:
                (?:
                    \[(?P<ipv6host>[^/]+)\] |
                    (?P<ipv4host>[^/:]+)
                )?
                (?::(?P<port>[^/]*))?
            )?
            (?:/(?P<database>.*))?
            """,
        re.X,
    )

    m = pattern.match(name)
    if m is not None:
        components = m.groupdict()
        if components["database"] is not None:
            tokens = components["database"].split("?", 2)
            components["database"] = tokens[0]

            if len(tokens) > 1:
                query = {}

                for key, value in parse_qsl(tokens[1]):
                    if key in query:
                        query[key] = to_list(query[key])
                        query[key].append(value)
                    else:
                        query[key] = value
            else:
                query = None
        else:
            query = None
        components["query"] = query

        if components["username"] is not None:
            components["username"] = _rfc_1738_unquote(components["username"])

        if components["password"] is not None:
            components["password"] = _rfc_1738_unquote(components["password"])

        ipv4host = components.pop("ipv4host")
        ipv6host = components.pop("ipv6host")
        components["host"] = ipv4host or ipv6host
        name = components.pop("name")
        return URL(name, **components)
    else:
        return None


def _rfc_1738_quote(text):
    return re.sub(r"[:@/]", lambda m: "%%%X" % ord(m.group(0)), text)


def _rfc_1738_unquote(text):
    return unquote(text)


def _parse_keyvalue_args(name):
    m = re.match(r"(\w+)://(.*)", name)
    if m is not None:
        (name, args) = m.group(1, 2)
        opts = dict(parse_qsl(args))
        return URL(name, *opts)
    else:
        return None


def generate_uuid():
    return str(uuid.uuid1()).replace("-", "")



# #!/usr/bin/env python
# # -*- coding: utf-8 -*-
# # @Time    : 2019-12-10 16:55
# # @Author  : Luke
# # @File    : ihdfs.py
# # @Desc    : hdfs connection
# import os
# import hdfs3
# from . import utils
# from . import connection
# from pathlib2 import Path
# import datetime
#
#
# class HDFSConnection(connection.Connection):
#     def __init__(self):
#         super().__init__()
#         # self.config = configparser.ConfigParser()
#         # self.config.read("imlp-notebook.conf", encoding="utf-8")
#         self.connection = self.connect()
#         self.user_id = os.getenv("USER_ID")
#         self.workspace_id = os.getenv("WORKSPACE_ID")
#
#     def connect(self, host=None, port=None, hdfs_ha_enable=None, user=None, kerberos=False):
#         if host is None:
#             # host = self.config.get("hdfs", "HDFS_HOST")
#             host = os.getenv("HDFS_HOST")
#         if hdfs_ha_enable is None:
#             # hdfs_ha_enable = self.config.getboolean("hdfs", 'HDFS_HA_ENABLE')
#             hdfs_ha_enable = True if os.getenv("HDFS_HA_ENABLE").lower() == "true" else False
#         if hdfs_ha_enable:
#             # kerberos = self.config.get("hdfs", "HDFS_KERBEROS")
#             kerberos = True if os.getenv("HDFS_KERBEROS").lower() == "true" else False
#             if kerberos:
#                 connection = hdfs3.HDFileSystem(host=host, pars={"hadoop.security.authentication": "kerberos"})
#             else:
#                 if user is None:
#                     # user = self.config.get("hdfs", "HDFS_USER")
#                     user = os.getenv("HDFS_USER")
#                 connection = hdfs3.HDFileSystem(host=host, user=user)
#         else:
#             if port is None:
#                 # port = self.config.getint("hdfs", "HDFS_PORT")
#                 port = int(os.getenv("HDFS_PORT"))
#             if kerberos:
#                 connection = hdfs3.HDFileSystem(host=host, port=port,
#                                                      pars={"hadoop.security.authentication": "kerberos"})
#             else:
#                 if user is None:
#                     # user = self.config.get("hdfs", "HDFS_USER")
#                     user = os.getenv("HDFS_USER")
#                 connection = hdfs3.HDFileSystem(host=host, port=port, user=user)
#         self.closed = False
#         return connection
#
#     def disconnect(self):
#         self.connection.disconnect()
#         self.closed = True
#
#     def get_root_path(self):
#         # return self.config.get("hdfs", "HDFS_ROOT_PATH")
#         if os.getenv("HDFS_ROOT_PATH") is None:
#             return "/imlp"
#         else:
#             return os.getenv("HDFS_ROOT_PATH")
#
#     def get_internal_path(self):
#         # return self.config.get("hdfs", "HDFS_ROOT_PATH")
#         if os.getenv("HDFS_ROOT_PATH") is None:
#             return "/imlp"
#         else:
#             return os.getenv("HDFS_ROOT_PATH")
#
#     def open_file(self, path):
#         """
#         open file and read as StringIO on HDFS
#         @param path: data path
#         @return: StringIO
#         """
#         from io import StringIO
#         with self.connection.open(path) as f:
#             data = StringIO(str(f.read(), "utf-8"))
#         return data
#
#     def upload_df(self, df, dataset_name, dataset_desc):
#         """
#         upload local Pandas.DataFrame as csv file to HDFS and show in Dataset module
#         @param df: Pandas.DataFrame
#         @param dataset_name: name to show in Dataset module
#         @param dataset_desc: description to show in Dataset module
#         @return: "success" or None
#         """
#
#         if not dataset_name.endswith(".csv"):
#             dataset_name += ".csv"
#         try:
#             # estimate file size
#             dataset_size = self._humanbytes(df.memory_usage(index=True).sum())
#             # determine file path
#             dst = self.get_root_path() + "/" + self.user_id + "/dataset/" + utils.generate_uuid()
#             parent_dir = str(Path(dst).parent)
#             if not self.connection.exists(parent_dir):
#                 self.connection.makedirs(parent_dir)
#             # save as csv file
#             with self.connection.open(dst, "wb") as writer:
#                 df.to_csv(writer, index=False, encoding='utf-8')
#
#             # insert a record to ABC_DATASET
#             from . import imysql
#             mysql_connection = imysql.MySQLConnection().connection
#             dataset_id = utils.generate_uuid()
#             resource_dir_id = utils.generate_uuid()
#             insert_dataset_statement = """
#                         INSERT INTO ABC_DATASET (DATASET_ID,WORKSPACE_ID,USER_ID,DATASET_NAME,DATA_DESC,DATASET_TYPE,DATASET_SIZE,DATASET_PATH,RESOURCE_DIR_ID,IS_ACTIVE,CREATOR,MODIFIER,CREATE_TIME,UPDATE_TIME,START_TIME,END_TIME)
#                         values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)
#                         """
#             try:
#                 with mysql_connection.cursor() as cursor:
#                     current_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
#                     row_count = cursor.execute(insert_dataset_statement,
#                                                [dataset_id, self.workspace_id, self.user_id, dataset_name, dataset_desc, "csv",
#                                                 dataset_size, dst, resource_dir_id, "1", self.user_id, self.user_id,
#                                                 current_time,
#                                                 current_time, current_time, current_time])
#                 mysql_connection.commit()
#                 if row_count > 0:
#                     # insert a record to ABC_RESOURCE_DIR
#                     insert_resource_dir_statement = """
#                                 INSERT INTO ABC_RESOURCE_DIR (ITEM_ID,WORKSPACE_ID,ITEM_NAME,ITEM_ORDER,PARENT_ID,ITEM_TYPE,IS_LEAF,RESOURCE_ID,IS_SYS,IS_READONLY,IS_DISPLAY,IS_ACTIVE,CREATOR,MODIFIER,CREATE_TIME,UPDATE_TIME,START_TIME,END_TIME,ITEM_DESC,USER_ID)
#                                 values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)
#                                 """
#                     try:
#                         with mysql_connection.cursor() as cursor:
#                             current_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
#                             row_count = cursor.execute(insert_resource_dir_statement,
#                                                        [resource_dir_id, self.workspace_id, dataset_name, 1, "-1",
#                                                         "1", "1", dataset_id, "1", "1", "1", "1", self.user_id,
#                                                         self.user_id, current_time, current_time, current_time,
#                                                         current_time, "", self.user_id])
#                             mysql_connection.commit()
#                             return "success"
#                     except Exception as e:
#                         print(e)
#                         return None
#                 else:
#                     return None
#             except Exception as e:
#                 print(e)
#                 return None
#         except Exception as e:
#             print(e)
#             return None
#
#     def open_model(self, path):
#         """
#         load a sklearn model from HDFS path
#         @param path: path copied from Model module
#         @return: model and meta
#         """
#
#         # specify model and meta location
#         model_path = str(Path(path) / "data")
#         meta_path = str(Path(path) / "metadata")
#         import joblib
#         model = None
#         meta = None
#         # load the model
#         with self.connection.open(model_path, "rb") as model_reader:
#             try:
#                 model = joblib.load(model_reader)
#             except Exception as e:
#                 print(e)
#         # load the meta
#         with self.connection.open(meta_path, "rb") as json_reader:
#             try:
#                 import json
#                 meta = json.load(json_reader)
#             except Exception as e:
#                 print(e)
#
#         return model, meta
#
#     def upload_model(self, df, model, model_name, model_desc):
#         """
#         save sklearn model to inner storage and show in Model module
#         @param df: training DataFrame
#         @param model: sklearn model
#         @param model_name: name to show in Model module
#         @param model_desc: description to show in Model module
#         """
#
#         # form the model meta with provided training DataFrame
#         meta = self._form_model_meta(df)
#         # determine save path
#         dst = self.get_root_path() + "/" + self.user_id + "/model/" + utils.generate_uuid()
#         if not self.connection.exists(dst):
#             self.connection.makedirs(dst)
#         import joblib
#         import json
#         # save the model
#         with self.connection.open(dst + "/" + "data", "wb") as writer:
#             joblib.dump(model, writer)
#         # save the meta
#         with self.connection.open(dst + "/" + "meta", "wb") as writer:
#             json.dump(meta, writer)
#
#         # insert a record to ABC_MODEL
#         from . import imysql
#         mysql_connection = imysql.MySQLConnection().connection
#         model_id = utils.generate_uuid()
#         resource_dir_id = utils.generate_uuid()
#         insert_model_statement = """
#                     INSERT INTO ABC_MODEL (MODEL_ID,WORKSPACE_ID,USER_ID,MODEL_NAME,MODEL_DESC,DEPLOY_ENGINE,MODEL_PATH,MODEL_TYPE,RESOURCE_DIR_ID,IS_ACTIVE,CREATOR,MODIFIER,CREATE_TIME,UPDATE_TIME,START_TIME,END_TIME,MODEL_METADATA)
#                     values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)
#                     """
#         try:
#             with mysql_connection.cursor() as cursor:
#                 current_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
#                 row_count = cursor.execute(insert_model_statement,
#                                            [model_id, self.workspace_id, self.user_id, model_name, model_desc, "",
#                                             dst, "1", resource_dir_id, "1", self.user_id, self.user_id,
#                                             current_time, current_time, current_time, current_time, json.dumps(meta)])
#             mysql_connection.commit()
#             if row_count > 0:
#                 # insert a record to ABC_RESOURCE_DIR
#                 insert_resource_dir_statement = """
#                             INSERT INTO ABC_RESOURCE_DIR (ITEM_ID,WORKSPACE_ID,ITEM_NAME,ITEM_ORDER,PARENT_ID,ITEM_TYPE,IS_LEAF,RESOURCE_ID,IS_SYS,IS_READONLY,IS_DISPLAY,IS_ACTIVE,CREATOR,MODIFIER,CREATE_TIME,UPDATE_TIME,START_TIME,END_TIME,ITEM_DESC,USER_ID)
#                             values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)
#                             """
#                 try:
#                     with mysql_connection.cursor() as cursor:
#                         current_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
#                         row_count = cursor.execute(insert_resource_dir_statement,
#                                                    [utils.generate_uuid(), self.workspace_id, model_name, 1, "-1",
#                                                     "0", "1", model_id, "1", "1", "1", "1", self.user_id,
#                                                     self.user_id, current_time, current_time, current_time,
#                                                     current_time, "", self.user_id])
#                         mysql_connection.commit()
#                         return "success"
#                 except Exception as e:
#                     print(e)
#                     return None
#             else:
#                 return None
#         except Exception as e:
#             print(e)
#             return None



#!/usr/bin/env python 
# -*- coding: utf-8 -*-
# @Time    : 2019-12-10 16:55
# @Author  : Luke
# @File    : model.py
# @Desc    : model utils

import os


class Model:
    """
    utils for connecting with Model module
    与平台模型模块交互的工具类
    Example:
        >>>import imlp
        >>>imodel = imlp.model.Model()
        >>>mymodel, model_meta = imodel.load_model("{{path}}")
    """
    def __init__(self):
        storage_type = os.getenv("STORAGE_TYPE")
        self.connection = None
        if storage_type.lower() == "minio":
            from . import iminio
            self.connection = iminio.MinIOConnection()
        elif storage_type.lower() == "mysql":
            from . import imysql
            self.connection = imysql.MySQLConnection()

    def save_model(self, df, model, model_name, model_desc=""):
        """
        save Scikit-Learn model to inner storage and show in Model module
        将Notebook中训练的Scikit-Learn模型保存到模型模块，需提供训练集的Pandas.DataFrame以保存元数据
        Example:
            >>>import imlp
            >>>imodel = imlp.model.Model()
            >>>imodel.save_model(df, cls, "sample_model", "sample_model_description")
        @param df: training DataFrame
        @param model: scikit-learn model
        @param model_name: name to show in Model module
        @param model_desc: model description shown in Model module
        """
        if self.connection.closed:
            self.__init__()
        import pandas as pd
        if not isinstance(df, pd.DataFrame):
            print("Failed! Please provide a pandas DataFrame as the first parameter")
        else:
            try:
                from sklearn.utils.estimator_checks import check_estimator
                check_estimator(model)
                result = self.connection.upload_model(df, model, model_name, model_desc)
                if result is not None:
                    print("saved successfully")
                self.connection.disconnect()

            except TypeError as e:
                print("Failed! Given model is not a scikit-learn estimator, cannot save")

    def load_model(self, path):
        """
        load Scikit-Learn model from inner storage saved from Notebook
        从模型模块加载Scikit-Learn模型
        Example:
            >>>import imlp
            >>>imodel = imlp.model.Model()
            >>>mymodel, model_meta = imodel.load_model("{{path}}")
        @param path: path copied from Model module
        @return: model and model meta
        """
        if self.connection.closed:
            self.__init__()
        try:
            model, meta = self.connection.open_model(path)
        except Exception as e:
            print("cannot load model")
            model = None
            meta = None
        finally:
            self.connection.disconnect()
        return model, meta



#!/usr/bin/env python 
# -*- coding: utf-8 -*-
# @Time    : 2019-12-10 15:00
# @Author  : Luke
# @File    : test.py
# @Desc    :
from imlp import dataset
from imlp import model
import os

def test_read_data():
    idataset = dataset.Dataset()
    df = idataset.read_csv("imlp/sysadmin/dataset/64832f7314c64455b1bc3948a94e5fc6")# zls_test_data/69b31a4ab1464b18a3dfa151212f4c49")
    print(df.size)


def test_write_data():
    import pandas as pd
    df = pd.read_csv("/Users/petra/Downloads/Iris.csv", sep=";")
    idataset = dataset.Dataset()
    idataset.save_dataset(df, "test_save2.csv")


def test_write_model():
    from sklearn.linear_model import LogisticRegression
    import pandas as pd
    df = pd.read_csv("/Users/petra/Downloads/Iris.csv", sep=";")
    y = df["label"]
    X = df.drop(["label", "id"], axis=1)
    clf = LogisticRegression(random_state=0).fit(X, y)
    imodel = model.Model()
    imodel.save_model(X, clf, "test_model")
    
def test_load_model():
    import pandas as pd
    df = pd.read_csv("/Users/petra/Downloads/Iris.csv", sep=";")
    X = df.drop(["id", "label"], axis=1)
    imodel = model.Model()
    modela, meta = imodel.load_model("imlp/sysadmin/model/f0261f86903011eaa32cf01898ece9c8")
    if modela is not None:
        print(modela.predict(X))

def test_doc():
    print(help(model))

if __name__ == "__main__":
    os.environ["STORAGE_TYPE"] = "minio"
    os.environ["USER_ID"] = "sysadmin"
    os.environ["WORKSPACE_ID"] = "a"
    os.environ["MINIO_ENDPOINT"] = "10.111.25.27:9000"
    os.environ["MINIO_ACCESSKEY"] = "minioadmin"
    os.environ["MINIO_SECRETKEY"] = "minioadmin"
    os.environ["MINIO_BUCKET"] = "imlp"
    os.environ[
        "MYSQL_CONNECTION"] = "mysql://root:Analysis321!@10.111.24.205:3306/imlp-abc-test"

    # test_read_data()
    # test_write_data()
    # test_write_model()
    test_load_model()
    # test_doc()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2019-12-10 16:55
# @Author  : Luke
# @File    : imysql.py
# @Desc    : mysql connection
from . import connection
import os
import pymysql
from . import utils
import datetime


class MySQLConnection(connection.Connection):
    def __init__(self):
        super().__init__()
        self.connection = self.connect()
        self.user_id = os.getenv("USER_ID")
        self.workspace_id = os.getenv("WORKSPACE_ID")
        self.data_table = "ABC_STORE_DATA"
        self.model_table = "ABC_STORE_MODEL"

    def connect(self, **kwargs):
        # config = configparser.ConfigParser()
        # config.read(Path(os.path.realpath(__file__)).parent / "imlp-notebook.conf", encoding="utf-8")
        connection_string = os.getenv("MYSQL_CONNECTION")
        if connection_string is None:
            raise RuntimeError(
                "no proper database connection information provided")
        url = utils.make_url(os.getenv("MYSQL_CONNECTION"))

        try:
            # connection = pymysql.connect(host=config.get("mysql", "host"),
            #                              port=int(config.get("mysql", "port")),
            #                              user=config.get("mysql", "username"),
            #                              password=config.get("mysql", "password"),
            #                              db=config.get("mysql", "db"), **kwargs)
            connection = pymysql.connect(host=url.host,
                                         port=int(url.port),
                                         user=url.username,
                                         password=url.password,
                                         db=url.database, **kwargs)
            self.closed = False
        except Exception as e:
            raise RuntimeError(
                "no proper database connection information provided")

        return connection

    def disconnect(self):
        self.connection.close()
        self.closed = True

    def get_root_path(self):
        return "mysql"

    def get_internal_path(self):
        return "mysql"

    def open_file(self, path):
        """
        open file and read as StringIO on MySQL
        @param path: data path
        @return: StringIO
        """
        temp = path.split("/")
        column = "data"
        table_name = self.data_table
        id = temp[1]
        if len(temp) > 2:
            table_name = temp[-3]
            column = temp[-1]
            id = temp[-2]

        return self.query_blob_to_bytes(table_name, id, column)

    def query_blob_to_bytes(self, table_name, id, column):
        """
        query blob column and transform to StringIO
        @param table_name:
        @param id:
        @param column:
        @return:
        """
        query_statement = "SELECT " + column + " FROM " + table_name + " WHERE ID= %s"
        blob = None
        from io import StringIO
        try:
            with self.connection.cursor() as cursor:
                cursor.execute(query_statement, [id])
                ret = cursor.fetchone()
                blob = ret[0]
                result = StringIO(str(blob, "utf-8"))
        except Exception as e:
            print(e)
            result = None
        return result

    def upload_df(self, df, dataset_name, dataset_desc):
        """
        upload local Pandas.DataFrame as csv file to MySQL and show in Dataset module
        @param df: Pandas.DataFrame
        @param dataset_name: name to show in Dataset module
        @param dataset_desc: description to show in Dataset module
        @return: "success" or None
        """

        if not dataset_name.endswith(".csv"):
            dataset_name += ".csv"
        try:
            # estimate file size
            dataset_size = self._humanbytes(df.memory_usage(index=True).sum())
            # determine file path
            result = self.upload_local_csv(df, dataset_name, dataset_desc, dataset_size)
        except Exception as e:
            print(e)
            result = None
        return result

    def upload_local_csv(self, df, dataset_name="", dataset_desc="", dataset_size=""):
        """
        upload local Pandas.DataFrame as csv file to MySQL and show in Dataset module
        @param df: Pandas.DataFrame
        @param dataset_name: name to show in Dataset module
        @param dataset_desc: description to show in Dataset module
        @param dataset_size: estimated data size
        @return: id or None
        """

        # determine file path
        dst = self.get_root_path() + "/" + self.user_id + \
            "/dataset/" + utils.generate_uuid()
        temp = dst.split("/")
        table = self.data_table
        id = temp[-1]
        # save as csv file to MySQL
        insert_path = self.insert_file(df, table, id)
        if insert_path is not None:
            # insert a record to ABC_DATASET
            resource_id = utils.generate_uuid()
            insert_dataset_statement = """
            INSERT INTO ABC_DATASET (DATASET_ID,WORKSPACE_ID,USER_ID,DATASET_NAME,DATA_DESC,DATASET_TYPE,DATASET_SIZE,DATASET_PATH,IS_ACTIVE,CREATOR,MODIFIER,CREATE_TIME,UPDATE_TIME,START_TIME,END_TIME)
            values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)
            """
            try:
                with self.connection.cursor() as cursor:
                    current_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
                    row_count = cursor.execute(insert_dataset_statement,
                                               [resource_id, self.workspace_id, self.user_id, dataset_name, dataset_desc, "csv",
                                                dataset_size, insert_path, "1", self.user_id, self.user_id, current_time,
                                                current_time, current_time, current_time])
                self.connection.commit()
                if row_count > 0:
                    # insert a record to ABC_RESOURCE_DIR
                    insert_resource_dir_statement = """
                    INSERT INTO ABC_RESOURCE_DIR (ITEM_ID,WORKSPACE_ID,ITEM_NAME,ITEM_ORDER,PARENT_ID,ITEM_TYPE,IS_LEAF,RESOURCE_ID,IS_SYS,IS_READONLY,IS_DISPLAY,IS_ACTIVE,CREATOR,MODIFIER,CREATE_TIME,UPDATE_TIME,START_TIME,END_TIME,ITEM_DESC,USER_ID)
                    values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)
                    """
                    try:
                        with self.connection.cursor() as cursor:
                            current_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
                            row_count = cursor.execute(insert_resource_dir_statement,
                                                       [utils.generate_uuid(), self.workspace_id, dataset_name, 1, "-1",
                                                        "1", "1", resource_id, "1", "1", "1", "1", self.user_id,
                                                        self.user_id, current_time, current_time, current_time,
                                                        current_time, "", self.user_id])
                            self.connection.commit()
                            return id
                    except Exception as e:
                        print(e)
                        return None
                else:
                    return None
            except Exception as e:
                print(e)
                return None
        else:
            return None

    def insert_file(self, df, table, id):
        insert_statement = "INSERT INTO " + \
            table + " (ID, DATA) values(%s, %s)"
        try:
            with self.connection.cursor() as cursor:
                row_count = cursor.execute(
                    insert_statement, [id, self._convert_to_binary_data(df)])
                self.connection.commit()
                if row_count > 0:
                    record = table + "/" + id
                else:
                    record = None
        except Exception as e:
            print(e)
            record = None

        return record

    @staticmethod
    def _convert_to_binary_data(df):
        from io import BytesIO, StringIO
        bytes_container = StringIO()
        df.to_csv(bytes_container, index=False)
        bytes_container.seek(0)  # update to enable reading
        binary_data = bytes_container.read()
        return binary_data

    @staticmethod
    def _convert_to_binary_model(model):
        from io import BytesIO
        import joblib
        bytes_container = BytesIO()
        joblib.dump(model, bytes_container)
        bytes_container.seek(0)  # update to enable reading
        binary_data = bytes_container.read()
        return binary_data

    def upload_model(self, df, model, model_name, model_desc):
        """
        save sklearn model to inner storage and show in Model module
        @param df: training DataFrame
        @param model: sklearn model
        @param model_name: name to show in Model module
        @param model_desc: model description shown in Model module
        """

        # form the model meta with provided training DataFrame
        meta = self._form_model_meta(df)
        # determine save path
        dst = self.get_root_path() + "/" + self.user_id + \
            "/model/" + utils.generate_uuid()
        temp = dst.split("/")
        table = self.model_table
        id = temp[-1]
        # save the model and meta
        insert_path = self.insert_model(model, meta, table, id)

        if insert_path is None:
            return None

        model_id = utils.generate_uuid()
        resource_dir_id = utils.generate_uuid()
        # insert a record to ABC_MODEL
        insert_model_statement = """
                    INSERT INTO ABC_MODEL (MODEL_ID,WORKSPACE_ID,USER_ID,MODEL_NAME,MODEL_DESC,DEPLOY_ENGINE,MODEL_PATH,MODEL_TYPE,RESOURCE_DIR_ID,IS_ACTIVE,CREATOR,MODIFIER,CREATE_TIME,UPDATE_TIME,START_TIME,END_TIME,MODEL_METADATA)
                    values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)
                    """
        try:
            with self.connection.cursor() as cursor:
                import json
                current_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
                row_count = cursor.execute(insert_model_statement,
                                           [model_id, self.workspace_id, self.user_id, model_name, model_desc, "", insert_path,
                                            "1", resource_dir_id, "1", self.user_id, self.user_id, current_time,
                                            current_time, current_time, current_time, json.dumps(meta)])
            self.connection.commit()
            if row_count > 0:
                # insert a record to ABC_RESOURCE_DIR
                insert_resource_dir_statement = """
                            INSERT INTO ABC_RESOURCE_DIR (ITEM_ID,WORKSPACE_ID,ITEM_NAME,ITEM_ORDER,PARENT_ID,ITEM_TYPE,IS_LEAF,RESOURCE_ID,IS_SYS,IS_READONLY,IS_DISPLAY,IS_ACTIVE,CREATOR,MODIFIER,CREATE_TIME,UPDATE_TIME,START_TIME,END_TIME,ITEM_DESC,USER_ID)
                            values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)
                            """
                try:
                    with self.connection.cursor() as cursor:
                        current_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
                        row_count = cursor.execute(insert_resource_dir_statement,
                                                   [utils.generate_uuid(), self.workspace_id, model_name, 1, "-1",
                                                    "0", "1", model_id, "1", "1", "1", "1", self.user_id,
                                                    self.user_id, current_time, current_time, current_time,
                                                    current_time, "", self.user_id])
                        self.connection.commit()
                except Exception as e:
                    print(e)
                    return None
            else:
                return None
        except Exception as e:
            print(e)
            return None

    def insert_model(self, model, meta, table, id):
        insert_statement = "INSERT INTO " + table + \
            " (ID, DATA, META) values(%s, %s, %s)"
        try:
            with self.connection.cursor() as cursor:
                import json
                row_count = cursor.execute(insert_statement,
                                           [id, self._convert_to_binary_model(model), json.dumps(meta)])
                self.connection.commit()
                if row_count > 0:
                    record = table + "/" + id
                else:
                    record = None
        except Exception as e:
            print(e)
            record = None

        return record

    def open_model(self, path):
        """
        load a sklearn model from HDFS path
        @param path: path copied from Model module
        @return: model and meta
        """
        temp = path.split("/")
        table_name = self.model_table
        id = temp[1]
        if len(temp) > 2:
            table_name = temp[-3]
            column = temp[-1]
            id = temp[-2]

        return self.query_model_and_meta(table_name, id)

    def query_model_and_meta(self, table_name, id):
        import joblib
        import json
        from io import BytesIO
        query_statement = "SELECT `DATA`,`META` FROM " + table_name + " WHERE ID= %s"
        model = None
        meta = None
        try:
            with self.connection.cursor() as cursor:
                cursor.execute(query_statement, [id])
                ret = cursor.fetchone()
                blob = ret[0]
                model = joblib.load(BytesIO(blob))
                meta = json.loads(ret[1])
        except Exception as e:
            print(e)

        return model, meta



# !/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2019-12-10 16:55
# @Author  : Luke
# @File    : dataset.py
# @Desc    : dataset utils

import os
import pandas as pd


class Dataset:
    """
    utils for connecting with Dataset module
    与平台数据集模块交互的工具类
    Example:
        >>>import imlp
        >>>idataset = imlp.dataset.Dataset()
        >>>df = idataset.read_csv({{path}})
    """
    def __init__(self):
        storage_type = os.getenv("STORAGE_TYPE")
        self.connection = None
        if storage_type.lower() == "minio":
            from . import iminio
            self.connection = iminio.MinIOConnection()
        elif storage_type.lower() == "mysql":
            from . import imysql
            self.connection = imysql.MySQLConnection()

    def read_csv(self, path, **kwargs):
        """
        read csv from inner storage
        从数据集模块读取csv类型数据集
        Example:
            >>>import imlp
            >>>idataset = imlp.dataset.Dataset()
            >>>df = idataset.read_csv({{path}})
        @param path: path copied from Dataset module
        @return: Pandas.DataFrame or None
        """
        if self.connection.closed:
            self.__init__()
        df = pd.read_csv(self.connection.open_file(path), encoding='utf-8', **kwargs)
        if df is not None:
            print(df.head())
            self.connection.disconnect()
            return df
        else:
            print("error reading dataset")
            self.connection.disconnect()
            return None

    def read_tsv(self, path, **kwargs):
        """
        read tsv from inner storage
        从数据集模块读取tsv类型数据集
        Example:
            >>>import imlp
            >>>idataset = imlp.dataset.Dataset()
            >>>df = idataset.read_tsv({{path}})
        @param path: path copied from Dataset module
        @return: Pandas.DataFrame or None
        """
        if self.connection.closed:
            self.__init__()
        df = pd.read_csv(self.connection.open_file(path), sep="\t", encoding='utf-8', **kwargs)
        if df is not None:
            print(df.head())
            self.connection.disconnect()
            return df
        else:
            print("error reading dataset")
            self.connection.disconnect()
            return None

    def read_excel(self, path, **kwargs):
        """
        read excel from inner storage
        从数据集模块读取csv类型数据集
        Example:
            >>>import imlp
            >>>idataset = imlp.dataset.Dataset()
            >>>df = idataset.read_excel({{path}})
        @param path: path copied from Dataset module
        @return: Pandas.DataFrame or None
        """
        if self.connection.closed:
            self.__init__()
        df = pd.read_excel(self.connection.open_file(path), encoding='utf-8', **kwargs)
        if df is not None:
            print(df.head())
            self.connection.disconnect()
            return df
        else:
            print("error reading dataset")
            self.connection.disconnect()
            return None

    def save_dataset(self, df, dataset_name, dataset_desc=""):
        """
        save Pandas.DataFrame to inner storage
        将Pandas.DataFrame以csv格式存储到数据集模块
        Example:
            >>>import imlp
            >>>import pandas as pd
            >>>idataset = imlp.dataset.Dataset()
            >>>df = pd.DataFrame({'col1': [1, 2], 'col2': [3, 4]})
            >>>idataset.save_dataset(df, "sample_dataset", "sample_dataset_description")
        @param df: Pandas.DataFrame dataframe to save
        @param dataset_name: name to show in Dataset module
        @param dataset_desc: description to show in Dataset module
        """
        if self.connection.closed:
            self.__init__()
        result = self.connection.upload_df(df, dataset_name, dataset_desc)
        if result is not None:
            print("saved successfully")
        else:
            print("encountered error")
        self.connection.disconnect()



from setuptools import setup

setup(
   name='imlp',
   version='1.0.1',
   description='IMLP SDK',
   author='bruce',
   author_email='zhaozhenxiu@inspur.com',
   packages=['imlp'],  #same as name
   install_requires=['pandas', 'scikit-learn', 'pymysql', 'pathlib2', 'minio']
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2019-12-10 16:55
# @Author  : Luke
# @File    : iminio.py
# @Desc    : minio connection
import os
from minio import Minio
from minio.error import (ResponseError, BucketAlreadyOwnedByYou,
                         BucketAlreadyExists)
from . import utils
from . import connection
from pathlib2 import Path
import datetime


class MinIOConnection(connection.Connection):
    def __init__(self):
        super().__init__()
        self.connection = self.connect()
        self.user_id = os.getenv("USER_ID")
        self.workspace_id = os.getenv("WORKSPACE_ID")
        self.bucket = os.getenv("MINIO_BUCKET", "imlp")
        try:
            if not self.connection.bucket_exists(self.bucket):
                self.connection.make_bucket(self.bucket)
        except ResponseError as err:
            print(err)

    def connect(self, host=None, port=None, hdfs_ha_enable=None, user=None, kerberos=False):
        endpoint = os.getenv("MINIO_ENDPOINT")
        access_key = os.getenv("MINIO_ACCESSKEY", "admin")
        secret_key = os.getenv("MINIO_SECRETKEY", "admin123")

        # Initialize client with an endpoint and access/secret keys.
        client = Minio(endpoint,
                            access_key=access_key,
                            secret_key=secret_key,
                            secure=False)
        self.closed = False
        return client

    def disconnect(self):
        self.connection = None
        self.closed = True

    def get_root_path(self):
        return "imlp"

    def get_internal_path(self):
        return "imlp"

    def open_file(self, path):
        """
        open file and read as StringIO on MinIO
        @param path: data path
        @return: StringIO
        """
        from io import StringIO
        with self.connection.get_object(self.bucket, path) as f:
            data = StringIO(str(f.read(), "utf-8"))
        return data

    def upload_df(self, df, dataset_name, dataset_desc):
        """
        upload local Pandas.DataFrame as csv file to minio and show in Dataset module
        @param df: Pandas.DataFrame
        @param dataset_name: name to show in Dataset module
        @param dataset_desc: description to show in Dataset module
        @return: "success" or None
        """

        if not dataset_name.endswith(".csv"):
            dataset_name += ".csv"
        try:
            # estimate file size
            dataset_size = self._humanbytes(df.memory_usage(index=True).sum())
            # determine file path
            dst = self.get_root_path() + "/" + self.user_id + "/dataset/" + utils.generate_uuid()
            from io import BytesIO
            csv_bytes = df.to_csv(index=False).encode('utf-8')
            csv_buffer = BytesIO(csv_bytes)

            self.connection.put_object(self.bucket,
                                   dst,
                                   data=csv_buffer,
                                   length=len(csv_bytes),
                                   content_type='application/csv')

            # insert a record to ABC_DATASET
            from . import imysql
            mysql_connection = imysql.MySQLConnection().connection
            dataset_id = utils.generate_uuid()
            resource_dir_id = utils.generate_uuid()
            insert_dataset_statement = """
                        INSERT INTO ABC_DATASET (DATASET_ID,WORKSPACE_ID,USER_ID,DATASET_NAME,DATA_DESC,DATASET_TYPE,DATASET_SIZE,DATASET_PATH,RESOURCE_DIR_ID,IS_ACTIVE,CREATOR,MODIFIER,CREATE_TIME,UPDATE_TIME,START_TIME,END_TIME)
                        values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)
                        """
            try:
                with mysql_connection.cursor() as cursor:
                    current_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
                    row_count = cursor.execute(insert_dataset_statement,
                                               [dataset_id, self.workspace_id, self.user_id, dataset_name, dataset_desc,
                                                "csv",
                                                dataset_size, dst, resource_dir_id, "1", self.user_id, self.user_id,
                                                current_time,
                                                current_time, current_time, current_time])
                mysql_connection.commit()
                if row_count > 0:
                    # insert a record to ABC_RESOURCE_DIR
                    insert_resource_dir_statement = """
                                INSERT INTO ABC_RESOURCE_DIR (ITEM_ID,WORKSPACE_ID,ITEM_NAME,ITEM_ORDER,PARENT_ID,ITEM_TYPE,IS_LEAF,RESOURCE_ID,IS_SYS,IS_READONLY,IS_DISPLAY,IS_ACTIVE,CREATOR,MODIFIER,CREATE_TIME,UPDATE_TIME,START_TIME,END_TIME,ITEM_DESC,USER_ID)
                                values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)
                                """
                    try:
                        with mysql_connection.cursor() as cursor:
                            current_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
                            row_count = cursor.execute(insert_resource_dir_statement,
                                                       [resource_dir_id, self.workspace_id, dataset_name, 1, "-1",
                                                        "1", "1", dataset_id, "1", "1", "1", "1", self.user_id,
                                                        self.user_id, current_time, current_time, current_time,
                                                        current_time, "", self.user_id])
                            mysql_connection.commit()
                            return "success"
                    except Exception as e:
                        print(e)
                        return None
                else:
                    return None
            except Exception as e:
                print(e)
                return None
        except Exception as e:
            print(e)
            return None

    def open_model(self, path):
        """
        load a sklearn model from minio path
        @param path: path copied from Model module
        @return: model and meta
        """

        # specify model and meta location
        model_path = str(Path(path) / "data")
        meta_path = str(Path(path) / "meta")
        import joblib
        from io import BytesIO
        model = None
        meta = None
        # load the model
        with self.connection.get_object(self.bucket, model_path) as model_reader:
            try:
                model = joblib.load(BytesIO(model_reader.read()))
            except Exception as e:
                print(e)
        # load the meta
        with self.connection.get_object(self.bucket, meta_path) as json_reader:
            try:
                import json
                meta = json.load(BytesIO(json_reader.read()))
            except Exception as e:
                print(e)

        return model, meta

    def upload_model(self, df, model, model_name, model_desc):
        """
        save sklearn model to inner storage and show in Model module
        @param df: training DataFrame
        @param model: sklearn model
        @param model_name: name to show in Model module
        @param model_desc: description to show in Model module
        """

        # form the model meta with provided training DataFrame
        meta = self._form_model_meta(df)
        # determine save path
        dst = self.get_root_path() + "/" + self.user_id + "/model/" + utils.generate_uuid()
        import joblib
        import json
        # save the model
        from io import BytesIO
        model_bytes_container = BytesIO()
        joblib.dump(model, model_bytes_container)
        model_bytes = model_bytes_container.tell()
        model_bytes_container.seek(0)  # update to enable reading

        self.connection.put_object(self.bucket,
                                   dst + "/" + "data",
                                   data=model_bytes_container,
                                   length=model_bytes)

        meta_bytes_container = BytesIO()
        meta_bytes_container.write(json.dumps(meta).encode("utf-8"))
        meta_bytes = meta_bytes_container.tell()
        meta_bytes_container.seek(0)  # update to enable reading
        # save the meta
        self.connection.put_object(self.bucket,
                                   dst + "/" + "meta",
                                   data=meta_bytes_container,
                                   length=meta_bytes)

        # insert a record to ABC_MODEL
        from . import imysql
        mysql_connection = imysql.MySQLConnection().connection
        model_id = utils.generate_uuid()
        resource_dir_id = utils.generate_uuid()
        insert_model_statement = """
                    INSERT INTO ABC_MODEL (MODEL_ID,WORKSPACE_ID,USER_ID,MODEL_NAME,MODEL_DESC,DEPLOY_ENGINE,MODEL_PATH,MODEL_TYPE,RESOURCE_DIR_ID,IS_ACTIVE,CREATOR,MODIFIER,CREATE_TIME,UPDATE_TIME,START_TIME,END_TIME,MODEL_METADATA)
                    values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)
                    """
        try:
            with mysql_connection.cursor() as cursor:
                current_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
                row_count = cursor.execute(insert_model_statement,
                                           [model_id, self.workspace_id, self.user_id, model_name, model_desc, "",
                                            dst, "1", resource_dir_id, "1", self.user_id, self.user_id,
                                            current_time, current_time, current_time, current_time, json.dumps(meta)])
            mysql_connection.commit()
            if row_count > 0:
                # insert a record to ABC_RESOURCE_DIR
                insert_resource_dir_statement = """
                            INSERT INTO ABC_RESOURCE_DIR (ITEM_ID,WORKSPACE_ID,ITEM_NAME,ITEM_ORDER,PARENT_ID,ITEM_TYPE,IS_LEAF,RESOURCE_ID,IS_SYS,IS_READONLY,IS_DISPLAY,IS_ACTIVE,CREATOR,MODIFIER,CREATE_TIME,UPDATE_TIME,START_TIME,END_TIME,ITEM_DESC,USER_ID)
                            values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)
                            """
                try:
                    with mysql_connection.cursor() as cursor:
                        current_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
                        row_count = cursor.execute(insert_resource_dir_statement,
                                                   [utils.generate_uuid(), self.workspace_id, model_name, 1, "-1",
                                                    "0", "1", model_id, "1", "1", "1", "1", self.user_id,
                                                    self.user_id, current_time, current_time, current_time,
                                                    current_time, "", self.user_id])
                        mysql_connection.commit()
                        return "success"
                except Exception as e:
                    print(e)
                    return None
            else:
                return None
        except Exception as e:
            print(e)
            return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2019-12-10 16:55
# @Author  : Luke
# @File    : connection.py
# @Desc    : connection class


class Connection:
    def __init__(self):
        self.connection = None
        self.closed = True

    def connect(self, **kwargs):
        pass

    def disconnect(self):
        pass
    
    def get_root_path(self):
        pass

    def get_internal_path(self):
        pass

    def open_file(self, path):
        pass

    def upload_df(self, df, dataset_name, dataset_desc):
        pass

    def open_model(self, path):
        pass

    def upload_model(self, df, model, model_name, model_desc):
        pass

    def _humanbytes(self, B):
        # Return the given bytes as a human friendly KB, MB, GB, or TB string
        B = float(B)
        KB = float(1024)
        MB = float(KB ** 2)  # 1,048,576
        GB = float(KB ** 3)  # 1,073,741,824
        TB = float(KB ** 4)  # 1,099,511,627,776

        if B < KB:
            return '{0} {1}'.format(B, 'Bytes' if 0 == B > 1 else 'Byte')
        elif KB <= B < MB:
            return '{0:.2f}K'.format(B / KB)
        elif MB <= B < GB:
            return '{0:.2f}M'.format(B / MB)
        elif GB <= B < TB:
            return '{0:.2f}G'.format(B / GB)
        elif TB <= B:
            return '{0:.2f}T'.format(B / TB)

    def _form_model_meta(self, df):
        from collections import OrderedDict
        return df.dtypes.apply(lambda x: x.name).to_dict(into=OrderedDict)
